<?php

declare(strict_types=1);

namespace Phant\Auth\Domain\Entity;

use Phant\Error\NotCompliant;

final class SslKey
{
    public function __construct(
        public readonly string $private,
        public readonly string $public
    ) {
    }

    public function encrypt(string $data): string
    {
        try {
            $success = openssl_private_encrypt(
                $data,
                $encryptedData,
                $this->private
            );
        } catch (\Exception $e) {
            $success = false;
        }

        if (!$success) {
            throw new NotCompliant('Encryption invalid, verify private key');
        }

        return $encryptedData;
    }

    public function decrypt(string $encryptedData): string
    {
        try {
            $success = openssl_public_decrypt(
                $encryptedData,
                $data,
                $this->public
            );
        } catch (\Exception $e) {
            $success = false;
        }

        if (!$success) {
            throw new NotCompliant('Decryption invalid, verify encrypted data or public key');
        }

        return $data;
    }
}
